import os
import sys

from django.core.management import execute_from_command_line


def runserver():
    appPath = os.path.join(os.path.abspath(__file__))
    appPath = os.path.dirname(appPath)
    sys.path.append(appPath)
    sys.path.append(os.path.join(appPath, '..', '..')) # Required to find mosamaticdesktop packages
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'aiservice.settings')
    os.chdir(appPath)
    print('##############################################################################')
    print('#          M O S A M A T I C  -  A I  S E R V I C E                          #')
    print('##############################################################################')
    execute_from_command_line(['manage.py', 'makemigrations'])
    execute_from_command_line(['manage.py', 'migrate'])
    execute_from_command_line(['manage.py', 'runserver', '0.0.0.0:8000'])


if __name__ == "__main__":
    runserver()



from django.shortcuts import render
from django.contrib.auth import get_user_model
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated, AllowAny, IsAdminUser
from rest_framework.response import Response
from rest_framework import status

from .serializers import UserRegistrationSerializer

from mosamaticdesktop.tasks.taskwidgetmanager import TaskWidgetManager

User = get_user_model()


@api_view(['GET'])
@permission_classes([AllowAny])
def api(request):
    return render(request, 'api.html')


@api_view(['POST'])
@permission_classes([AllowAny])
def registerUser(request):
    serializer = UserRegistrationSerializer(data=request.data)
    if serializer.is_valid():
        serializer.save()
        return Response({"message": "User created successfully"}, status=status.HTTP_201_CREATED)
    return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


@api_view(['GET'])
@permission_classes([IsAuthenticated])
def tasks(request):
    """
    This endpoint returns a list of task configurations. Each task configuration
    contains the task name and its parameters (names and types). 
    """
    manager = TaskWidgetManager(None)
    taskNames = manager.taskNames()
    taskWidgets = manager.taskWidgets()
    data = {}
    for taskName in taskNames:
        data[taskName] = []
        for taskParameter in taskWidgets[taskName].taskParameterWidgets():
            data[taskName].append(taskParameter.toDict())
    return Response({'tasks': data}, status=status.HTTP_200_OK)


@api_view(['GET'])
@permission_classes([IsAuthenticated])
def task(request, name):
    manager = TaskWidgetManager(None)
    taskWidget = manager.taskWidget(name=name)
    data = {name: []}
    for taskParameter in taskWidget.taskParameterWidgets():
        data[name].append(taskParameter.toDict())
    return Response(data, status=status.HTTP_200_OK)


@api_view(['POST'])
@permission_classes([IsAuthenticated])
def createTask(request, name):
    pass


@api_view(['POST'])
@permission_classes([IsAuthenticated])
def uploadDataToTask(request, taskId):
    pass


@api_view(['GET'])
@permission_classes([IsAuthenticated])
def taskStatus(request, taskId):
    pass


@api_view(['GET'])
@permission_classes([IsAuthenticated])
def downloadResultsFromTask(request, taskId):
    pass


from django.urls import path
from rest_framework_simplejwt.views import TokenRefreshView, TokenObtainPairView

from .views import api, registerUser, tasks, task, createTask, uploadDataToTask, taskStatus, downloadResultsFromTask


urlpatterns = [
    path('', api),
    path('api/token', TokenObtainPairView.as_view()),
    path('api/token/refresh', TokenRefreshView.as_view()),
    path('api/users/register', registerUser),
    path('api/tasks/', tasks),
    path('api/tasks/<str:name>', task),
    path('api/tasks/create', createTask),
    path('api/tasks/<str:taskId>/upload', uploadDataToTask),
    path('api/tasks/<str:taskId>/status', taskStatus),
    path('api/tasks/<str:taskId>/download', downloadResultsFromTask),
]


from django.contrib.auth import get_user_model
from django.contrib.auth.hashers import make_password
from rest_framework import serializers

User = get_user_model()


class UserRegistrationSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('username', 'password', 'email', 'first_name', 'last_name')
        extra_kwargs = {'password': {'write_only': True}}

    def create(self, validated_data):
        user = User.objects.create_user(
            username=validated_data['username'],
            email=validated_data['email'],
            first_name=validated_data['first_name'],
            last_name=validated_data['last_name'],
            password=validated_data['password']
        )
        return user